//
//  GDPRHelper.swift
//  sdk-demo-ios
//
//  Created by Created by Smaato Inc on 04/12/2019.
//  Copyright © 2019 Smaato Inc. All rights reserved.
//

import UIKit

protocol GDPRHelperDelegate : class {
    func needsUpdateGDPRInformation()
}

class GDPRHelper: NSObject {
    private let consentURL = "https://demofiles.smaato.com/cmp/index.html"
    private lazy var gdprAPI: CMPConsentToolAPI = {
        let api = CMPConsentToolAPI()
        api.subjectToGDPR = .unknown
        api.cmpPresent = true
        return api
    }()
    weak var delegate : GDPRHelperDelegate?
    
    func shouldShowGDPRTool() -> Bool {
        let subject = gdprAPI.subjectToGDPR;
        if ((subject == .yes || subject == .unknown) && (gdprAPI.consentString ?? "").isEmpty) {
            return true;
        }
        
        return false;
    }
    
    func gdprInfo() -> String {
        var str = String()

        str += "SubjectToGDPR: \(gdprAPI.subjectToGDPR.rawValue)\n\n"
        str += "ConsentString: \(String(describing: gdprAPI.consentString))\n\n"
        str += "VendorConsentBitString: \(String(describing: gdprAPI.parsedVendorConsents))\n\n"
        str += "PurposeConsentBitString: \(String(describing: gdprAPI.parsedPurposeConsents))\n\n"
        str += "CmpPresent: \(gdprAPI.cmpPresent)\n\n"

        let vendorId: Int32 = 82
        let vendorConsent = gdprAPI.isVendorConsentGiven(for: vendorId)
        let consentString = vendorConsent ? "YES" : "NO"
        str += "Consent for vendor ID: \(vendorId) = \(consentString)\n\n"

        for purposeId: Int32 in 1...5 {
            let purposeConsent = gdprAPI.isPurposeConsentGiven(for: purposeId)
            let purposeString = purposeConsent ? "YES" : "NO"
            str += "Consent for purpose ID: \(purposeId) = \(purposeString)\n"
        }

        return str
    }
    
    func showGDPRTool(onViewController parentVC: UIViewController, completion: (() -> Void)? = nil) {
        let toolVC = CMPConsentToolViewController()
        toolVC.consentToolURL = URL(string: consentURL)
        toolVC.consentToolAPI = gdprAPI
        toolVC.delegate = self
        parentVC.present(toolVC, animated: true, completion: completion)
    }
}

extension GDPRHelper: CMPConsentToolViewControllerDelegate {

    //MARK: - CMPConsentToolViewControllerDelegate
    func consentToolViewController(_ consentToolViewController: CMPConsentToolViewController!, didReceiveConsentString consentString: String!) {
        delegate?.needsUpdateGDPRInformation()
    }

}
